/*
 *  STYLE 1
 */

 *::-webkit-scrollbar-track
 {
     -webkit-box-shadow: inset 0 0 3px rgba(0,0,0,0.3);
     border-radius: 10px;
     background-color: rgba(47,49,54,0.3);
 }
 
 *::-webkit-scrollbar
 {
     width: 12px;
     background-color: rgba(47,49,54,0.3);
 }
 
 *::-webkit-scrollbar-thumb
 {
     border-radius: 10px;
     -webkit-box-shadow: inset 0 0 3px rgba(0,0,0,.3);
     background-color: rgba(66,66,66,0.6);
 }

html, body {
    background-color: #2f3136;
    color: white;
    width: 100%;
    overflow-x: hidden;
}

.modal-content {
    background-color: #2f3136;
    color: white;
}

html, body, #root {
    height: 100vh;
}

.main > .row {
    flex: 1;
    position: relative;
    margin-right: 0;
    max-height: 100%;
}

.main-col {
    min-width: min-content;
    flex: 1;
}

a {
    color: #3899D8;
}

select, input {
    background-color: #212121;
    border: 1px solid #17181b;
    border-radius: 4px;
    color: #f0f0f0;
}

.btn-group-xs > .btn,
.btn-xs {
    padding: 0.125rem 0.25rem;
    font-size: 0.75rem;
    line-height: 0.5;
    border-radius: 0.2rem;
}

.btn-popup, .btn-popup:hover {
    color: #fff;
    cursor: pointer;
    padding: 0;
}

.btn-popup .material-icons {
    font-size: 16px;
}

.popover, .popover-body {
    background-color: #212121;
    color: #fff;
}

.popover-body table {
    width: 100%;
}

.popover-header {
    background-color: #17181b;
    color: #fff;
    border-color: #2f3136;
}

.btn-icon {
    color: white;
    padding: .15rem .2rem;
    font-size: .75rem;
    line-height: .5;
    border-radius: .2rem;
}

.btn-icon .tool-label {
    vertical-align: baseline;
    line-height: 0.75rem;
    font-weight: bold;
}

.btn-icon .material-icons {
    font-size: 1rem;
    vertical-align: text-top;
}

.main {
    display: flex;
    height: 100vh;
    flex-direction: column;
}

.toolbar {
    background-color: #212121;
    padding: 0.25rem;
    display: flex;
}

.toolbar .badge {
    align-self: center;
    margin-right: 0.25rem;
}

.btn-icon {
    margin-right: 0.25rem;
}

#file_modded::before {
    content: ' ';
    flex: 1;
}

.open-path {
    flex: 1;
    height: 1.5rem;
    white-space: nowrap;
    overflow: hidden;
    text-overflow: ellipsis;
    padding: 0 0.33rem;
    font-size: 0.9rem;
    color: #e0e0e0;
}

.nav {
    background-color: #212121;
    border: 0;
}

.nav-item a {
    color: #e0e0e0;
    font-weight: 500;
    display: block;
    padding: 0.33rem 0.5rem;
}

.nav-item a.active {
    background-color: #2f3136;
    border-radius: 2px 2px 0 0;
}

.tab-content {
    padding: 0.5rem 1rem;
    height: 100%;
    overflow-y: auto;
}

.tab-pane {
    height: 100%;
}

.tab-pane > .row {
    height: 100%;
    overflow: hidden;
    padding: 0 1rem;
    flex-wrap: nowrap;
}

.info {
    overflow-y: auto;
    overflow-x: hidden;
}

.info > .row {
    flex-wrap: nowrap;
    margin: 0;
}

.info > .row > :first-child {
    margin-right: 1rem;
}

.prog_list {
    display: flex;
    flex: 0 !important;
    flex-direction: column;
    height: 100%;
    max-height: 100%;
    max-width: 30%;
    white-space: nowrap;
    margin-right: 1rem;
}

.items_list {
    flex: 1;
}

.list_buttons {
    margin-top: 0.5rem;
    margin-left: -0.25rem;
}

.list_buttons .btn {
    margin: 0 0.25rem;
    width: 1.5rem;
    height: 1.5rem;
    line-height: 0;
    font-weight: bold;
    padding: 0;
}

.params {
    width: 100%;    
}

.params td {
    padding: 1px 4px;
    white-space: nowrap;
}

.params select, .params input:not([type='checkbox']) {
    min-width: 3.5rem;
    width: 100%;
}

.params input:not([type='checkbox']) {
    padding: 1px 0.25rem;
}

.vec3 {
    display: inline-flex;
    margin: 0 -2.5px;
}

.vec3 input[type="number"] {
    flex-grow: 0;
    flex-shrink: 1;
    min-width: 0;
    display: inline-block;
    margin: 0 2.5px;
}

h2 {
    font-size: 1.25rem;
}

h3 {
    font-size: 1rem;
}

select {
    max-width: 100%;
}

.info-box {
    margin-bottom: 1rem;
    border-radius: 0.25rem;
    padding: 0.5rem 0.75rem;
    background: #232529;
}

.table {
    color: inherit;
}

.table td, .table th, .table thead th {
    border: 0;
}

.content {
    flex: 1;
    max-height: 100%;
    height: 100%;
    display: flex;
    flex-direction: column;
    overflow-y: hidden;
}

#treebar {
    background-color: #17181b;
    padding: 0.5rem 1.33rem;
    min-width: 10%;
    max-width: 21.875%;
    width: auto;
    transition: width 0.5s;
    overflow-x: auto;
}

.tree-root {
    margin: 0;
    padding: 0;
    list-style-type: none;
}

.tree-nested {
    margin: 0;
    padding-left: 12px;
    list-style-type: none;
    vertical-align: middle;
}

.tree-collapsed {
    display: none;
}

.tree-root li {
    white-space: nowrap;
}

.tree-root li a {
    cursor: pointer;
}

.caret {
    font-weight: bold;
    vertical-align: text-bottom;
    width: 0;
    margin-right: 12px;
    display: inline-block;
}

.caret:before {
    content: '-';
    width: 0;
}

.caret.up:before {
    content: '+'
}

.fab {
    border-radius: 50%;
    position: fixed;
    right: 15px;
    bottom: 15px;
    z-index: 999;
    width: 48px;
    height: 48px;
    padding: 0;
    font-size: 32px;
    font-weight: bold;
    box-shadow: 0 8px 16px rgba(0,0,0,0.19), 0 4px 4px rgba(0,0,0,0.23);
    transition: bottom 0.25s ease-out;
}

.fab-away {
    bottom: -90px;
}

#loader {
    top: 40%;
    width: 100vw;
    position: fixed;
}

#loader > div {
    margin: 0 auto;
}

.lds-ring {
    display: block;
    width: 64px;
    height: 64px;
  }
  .lds-ring div {
    box-sizing: border-box;
    display: block;
    position: absolute;
    width: 51px;
    height: 51px;
    margin: 6px;
    border: 6px solid #fff;
    border-radius: 50%;
    animation: lds-ring 1.2s cubic-bezier(0.5, 0, 0.5, 1) infinite;
    border-color: #fff transparent transparent transparent;
  }
  .lds-ring div:nth-child(1) {
    animation-delay: -0.45s;
  }
  .lds-ring div:nth-child(2) {
    animation-delay: -0.3s;
  }
  .lds-ring div:nth-child(3) {
    animation-delay: -0.15s;
  }
  @keyframes lds-ring {
    0% {
      transform: rotate(0deg);
    }
    100% {
      transform: rotate(360deg);
    }
  }
  